#!/usr/bin/env python
# -*- coding: utf-8 -*-

import copy
from globalStuff import *

# ----------------------------------------------------------------------
# 
class UndoHistory():
    def __init__(self, tables, relations):
        self.tables=tables
        self.relations=relations
        self.hist=[]                                                    # Format:  ( [ (,), ... ], [ ( "x", "y", [ (,), ...], [], ... ) ] )
        self.last_undo_point=None

    # ------------------------------------------------------------------
    # Setzt die Undo-Historie zurück.
    # Ist aufzurufen, wenn eine Datei neu geladen wird.
    def reset(self):
        self.hist=[]
        self.last_undo_point=None

    # ------------------------------------------------------------------
    # Speichert eine Kopie der derzeitigen Positionsdaten.
    # Ist bei MouseLeftDown aufzurufen.
    def prepare(self):
        self.last_undo_point=self.__getCurrentData()
        #print(self.last_undo_point)

    # ------------------------------------------------------------------
    # Liefert die Anzahl der zurücknehmbaren Änderungen.
    def length(self):
        return len(self.hist)

    # ------------------------------------------------------------------
    # Liefert zwei Listen mit den Tabellen- und Relations-Positionen.
    def __getCurrentData(self):
        ts=[]
        for t in self.tables:
            ts.append(t.pos)
        rs=[]
        for r in self.relations:
            sd=(r.border_start, r.border_end, copy.copy(r.line))        # zur Änderungserkennung braucht es eine Kopie der Liste - keinen Pointer
            rs.append(sd)
        return ts, rs

    # ------------------------------------------------------------------
    # Speichert die letzten Positionsdaten von veränderten Objekten,
    # sofern eine Änderung zwischen dem letzten Aufruf von
    # self.prepare() und dem aktuellen Stand erkannt wurde.
    # Ist bei MouseLeftUp aufzurufen.
    def trackModifications(self):
        if self.last_undo_point is None:
            print("prepare fehlt!")
            return
        modt=[]
        for i, t in enumerate(self.tables):
            if t.pos!=self.last_undo_point[0][i]:
                modt.append((t, self.last_undo_point[0][i]))
        modr=[]
        for i, r in enumerate(self.relations):
            bs, be, lst=self.last_undo_point[1][i]
            if r.border_start!=bs or r.border_end!=be or r.line!=lst:
                modr.append((r, self.last_undo_point[1][i]))
        if len(modt)==0 and len(modr)==0:
            return
        self.hist.append((modt, modr))
        self.last_undo_point=None

    # ------------------------------------------------------------------
    # Macht die letzte Änderung rückgängig.
    def lastModification(self):
        if len(self.hist)==0:
            return
        mt, mr=self.hist.pop()
        for t, p in mt:
            t.pos=p
        for r, d in mr:
            bs, be, l=d
            r.border_start=bs
            r.border_end=be
            r.line=l
